@@ -1,4 +1,4 @@
-# Copyright (c) 2018 Micro Focus or one of its affiliates.
+# Copyright (c) 2018-2019 Micro Focus or one of its affiliates.
 # Copyright (c) 2018 Uber Technologies, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -46,8 +46,8 @@ from .errors import (
     InternalError, NotSupportedError, OperationalError, ProgrammingError)
 
 # Main module for this library.
-__author__ = 'Uber Technologies, Inc'
-__copyright__ = 'Copyright (c) 2018 Micro Focus or one of its affiliates.'
+__author__ = 'Vertica'
+__copyright__ = 'Copyright (c) 2018-2019 Micro Focus or one of its affiliates.'
 __license__ = 'Apache 2.0'
 
 __all__ = ['Connection', 'PROTOCOL_VERSION', 'version_info', 'apilevel', 'threadsafety',
@@ -56,7 +56,7 @@ __all__ = ['Connection', 'PROTOCOL_VERSION', 'version_info', 'apilevel', 'thread
            'OperationalError', 'ProgrammingError']
 
 # The version number of this library.
-version_info = (0, 9, 1)
+version_info = (0, 9, 4)
 __version__ = '.'.join(map(str, version_info))
 
 # The protocol version (3.5) implemented in this library.
@@ -64,4 +64,9 @@ PROTOCOL_VERSION = 3 << 16 | 5
 
 apilevel = 2.0
 threadsafety = 1  # Threads may share the module, but not connections!
-paramstyle = 'named'  # WHERE name=:name
+
+# Accepted paramstyles are
+#   'qmark' = Question mark style, e.g. '...WHERE name=?'
+#   'named' = Named style, e.g. '...WHERE name=:name'
+#   'format' = ANSI C printf format codes, e.g. '...WHERE name=%s'
+paramstyle = 'named'
