@@ -1,4 +1,4 @@
-# Copyright (c) 2018 Micro Focus or one of its affiliates.
+# Copyright (c) 2018-2019 Micro Focus or one of its affiliates.
 # Copyright (c) 2018 Uber Technologies, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -37,6 +37,7 @@ from __future__ import print_function, division, absolute_import
 
 from datetime import date, datetime, time
 from decimal import Decimal
+from uuid import UUID
 import logging
 import os as _os
 import re
@@ -212,6 +213,24 @@ class CursorTestCase(VerticaPythonIntegrationTestCase):
             res_from_cur2 = cur2.fetchall()
             self.assertListOfListsEqual(res_from_cur2, [[2, 'bar']])
 
+    def test_copy_with_closed_file(self):
+        with tempfile.TemporaryFile() as tmpfile, self._connect() as conn:
+            if _os.name != 'posix' or _os.sys.platform == 'cygwin':
+                f = getattr(tmpfile, 'file')
+            else:
+                f = tmpfile
+
+            f.write(b"1,foo\n2,bar")
+            # move rw pointer to top of file
+            f.seek(0)
+
+            cur = conn.cursor()
+            f.close()
+            with self.assertRaisesRegexp(ValueError, 'closed file'):
+                cur.copy("COPY {0} (a, b) FROM STDIN DELIMITER ','".format(self._table),
+                          f)
+            cur.close()
+
     # unit test for #78
     def test_copy_with_data_in_buffer(self):
         with self._connect() as conn:
@@ -240,7 +259,7 @@ class CursorTestCase(VerticaPythonIntegrationTestCase):
             res = [[]]
             try:
                 cur.copy("COPY non_existing_tab(a, b) FROM STDIN DELIMITER ','", "FAIL")
-            except errors.Error as e:
+            except errors.MissingRelation as e:
                 cur.execute("SELECT 1;")
                 res = cur.fetchall()
 
@@ -259,7 +278,7 @@ class CursorTestCase(VerticaPythonIntegrationTestCase):
             try:
                 cur.copy("COPY {0} (a, b) FROM STDIN DELIMITER ',' ABORT ON ERROR".format(self._table),
                          "FAIL")
-            except errors.Error as e:
+            except errors.CopyRejected as e:
                 cur.execute("SELECT 1;")
                 res = cur.fetchall()
 
@@ -332,6 +351,11 @@ class CursorTestCase(VerticaPythonIntegrationTestCase):
             with self.assertRaises(errors.QueryError):
                 cur.execute("SELECT * FROM {0}_fail".format(self._table))
 
+            # generate a user-defined error message
+            err_msg = 'USER GENERATED ERROR: test error'
+            with self.assertRaisesRegexp(errors.QueryError, err_msg):
+                cur.execute("SELECT THROW_ERROR('test error')")
+
             # verify cursor still usable after errors
             cur.execute("SELECT a, b FROM {0} WHERE a = 1".format(self._table))
             res = cur.fetchall()
@@ -353,6 +377,8 @@ class CursorTestCase(VerticaPythonIntegrationTestCase):
 
                 # close and reopen cursor
                 cur.close()
+                with self.assertRaisesRegexp(errors.InterfaceError, 'Cursor is closed'):
+                    cur.execute("SELECT 1;")
                 cur = conn.cursor()
 
     def test_format_quote_unicode(self):
@@ -474,6 +500,43 @@ class SimpleQueryTestCase(VerticaPythonIntegrationTestCase):
                                                datetime(2018, 9, 7, 15, 38, 19, 769000),
                                                date(2018, 9, 7), time(13, 50, 9)]])
 
+    def test_binary_types(self):
+        with self._connect() as conn:
+            cur = conn.cursor()
+
+            # create test table
+            cur.execute("""CREATE TABLE {0} (
+                                a binary(1),
+                                b binary(3),
+                                c varbinary
+                            )
+                        """.format(self._table))
+
+            cur.execute("INSERT INTO {0} VALUES (:b, :s1, :s2)".format(self._table),
+                        {'b': b'x', 's1': b'xyz', 's2': 'abcde'})
+            conn.commit()
+
+            cur.execute("SELECT a, b, c FROM {0}".format(self._table))
+            res = cur.fetchall()
+            self.assertListOfListsEqual(res, [[b'x', b'xyz', b'abcde']])
+
+    def test_uuid_type(self):
+        with self._connect() as conn:
+            cur = conn.cursor()
+            x = UUID('{00010203-0405-0607-0809-0a0b0c0d0e0f}')
+            y = UUID('00000020-0000-0000-0000-100000000000')
+            z = UUID('00100010000000000000000000000000')
+
+            # create test table
+            cur.execute("CREATE TABLE {0} ( a uuid, b uuid, c uuid )".format(self._table))
+            cur.execute("INSERT INTO {0} VALUES (:u1, :u2, :u3)".format(self._table),
+                        {'u1': x, 'u2': y, 'u3': z})
+            conn.commit()
+
+            cur.execute("SELECT a, b, c FROM {0}".format(self._table))
+            res = cur.fetchall()
+            self.assertListOfListsEqual(res, [[str(x), str(y), str(z)]])
+
     # unit test for #74
     def test_nextset(self):
         with self._connect() as conn:
@@ -554,6 +617,14 @@ class SimpleQueryTestCase(VerticaPythonIntegrationTestCase):
             res = cur.fetchall()
             self.assertListOfListsEqual(res, [values])
 
+    def test_execute_parameters(self):
+        with self._connect() as conn:
+            cur = conn.cursor()
+            all_chars = u"".join(chr(i) for i in range(1, 128))
+            backslash_data = u"\\backslash\\ \\data\\\\"
+            cur.execute("SELECT :a, :b", parameters={"a": all_chars, "b": backslash_data})
+            self.assertEqual([all_chars, backslash_data], cur.fetchone())
+
 
 class SimpleQueryExecutemanyTestCase(VerticaPythonIntegrationTestCase):
     def setUp(self):
@@ -606,6 +677,20 @@ class SimpleQueryExecutemanyTestCase(VerticaPythonIntegrationTestCase):
     def test_executemany_utf8(self):
         self._test_executemany(self._table, [(1, u'a\xfc'), (2, u'bb')])
 
+    # test for #292
+    def test_executemany_autocommit(self):
+        with self._connect() as conn:
+            cur = conn.cursor()
+            cur.execute('SET SESSION AUTOCOMMIT TO off')
+            cur.execute('BEGIN')
+            cur.executemany("INSERT INTO {0} (a, b) VALUES (%s, %s)".format(self._table),
+                            ((None, 'foo'), [2, None], [3, 'bar']))
+            cur.execute('ROLLBACK')
+
+            cur.execute("SELECT count(*) FROM {0}".format(self._table))
+            res = cur.fetchone()[0]
+            self.assertEqual(res, 0)
+
     def test_executemany_null(self):
         seq_of_values_1 = ((None, 'foo'), [2, None])
         seq_of_values_2 = ({'a': None, 'b': 'bar'}, {'a': 4, 'b': None})
