@@ -1,4 +1,4 @@
-# Copyright (c) 2018 Micro Focus or one of its affiliates.
+# Copyright (c) 2018-2019 Micro Focus or one of its affiliates.
 # Copyright (c) 2018 Uber Technologies, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -41,9 +41,9 @@ portal exists. Execute doesn't cause RowDescription response message to be
 issued, so the frontend should issue Describe before issuing Execute, to ensure
 that it knows how to interpret the result rows it will get back.
 
-The Execute message specifies the portal name and a maximum result-row count
-(zero meaning "fetch all rows"). Currently, Vertica backend will ignore this
-result-row count and send all the rows regardless of what you put here.
+The Execute message specifies the portal name and a maximum result-row count.
+Currently, Vertica backend will ignore this result-row count and send all the
+rows regardless of what you put here.
 """
 
 from __future__ import print_function, division, absolute_import
